import java.io.*;

/**
* A class to provide communication facilities to the real robot arm
* @author Ho Ng
* @version 01/09/1998
*/
public class Armtty
{
    private int connecter;

    /**
    * purpose : send a command string to the serial port
    * @param command the command String
    * @return true (succeed) or false (fail)
    */
    private boolean sendCommand(String command)
    {
        if (connecter < 0)
        {
            System.out.println("Armtty : Call connect first");
            return false;
        }

	System.out.println("Armtty: Sending command " + command);
        if (send(connecter, command) == 0)
        {
            System.out.println("Armtty : Unable to send the command");
            return false;
        }
	System.out.println("Armtty: successful");

        return true;
    }

    /**
    * Default constructor
    * purpose : initialize the Armtty object
    */
    public Armtty()
    {
        connecter = -1;
    }

    /**
    * purpose : open the serial port to communicate with the robot arm
    * @return true (succeed) or false (fail)
    */
    public boolean connect()
    {
	System.out.println("Armtty: connecting");
        if (connecter >=  0)
        {
            System.out.println("Armtty : Call disconnect first");
            return false;
        }

	System.out.println("Armtty: initializing");
        connecter = init();
        if (connecter < 0)
        {
            System.out.println("Armtty : Could not open the serial port");
            return false;
        }

        String command = new String("NOHELP"); 
	System.out.println("Armtty: disabling HELP mode");
        sendCommand(command);
	System.out.println("Armtty: hack");
        sendCommand(command);    /* hack to wait */
	System.out.println("Armtty: done");
        return true;
    }

    /**
    * purpose : close the connection to the robot arm
    * @return true (succeed) or false (fail)
    */
    public boolean disconnect()
    {
        if (connecter < 0)
        {
            System.out.println("Armtty : Call connect first");
            return false;
        }

        if (quit(connecter) == 0)
        {
            System.out.println("Armtty : Could not close the serial port");
            return false;
        }

        connecter = -1;
        return true;
    }

    /**
    * purpose : send the command HOME to the port
    */
    public void home()
    {
        String command = new String("HOME");
        sendCommand(command);
        sendCommand(command);    /* hack to wait */
    }
 
    /**
    * purpose : send the command READY to the port
    */
    public void ready()
    {
        String command = new String("READY");
        sendCommand(command);
        sendCommand(command);    /* hack to wait */
    }
    
    /**
    * purpose : wait for the previous motion instruction to complete
    */
    public void finish()
    {
       String command = "FINISH";
       sendCommand(command);
       command = "DELAY 1";
       sendCommand(command);
    }
 
    /**
    * purpose : send the command MA to the port
    * @param waist waist angle measured from horizon
    * @param shoulder shoulder angle measured from horizon
    * @param elbow elbow measured from horizon
    * @param wrist wrist's bend angle measured from horizon
    * @param twist wrist's twist angle
    */
    public void ma(double waist, double shoulder, double elbow, double wrist,
                   double twist)
    {
        String command = String.format("MA %7.4f %7.4f %7.4f %7.4f %7.4f",
                           waist,shoulder,elbow,wrist,twist);
        sendCommand(command);
        sendCommand(command);    /* hack to wait */
    }

    /**
    * purpose : send the command GRIP to the port
    * @param width width between two fingers of the gripper
    */
    public void grip(double width)
    {
        String command = new String("GRIP " + width);
        sendCommand(command);
        sendCommand(command);    /* hack to wait */
    }

    /**
    * purpose : send command SPEED to the port
    * @param value speed of the arm
    */
    public void speed(int value)
    {
        String command = new String("SPEED " + value);
        sendCommand(command);
        sendCommand(command);    /* hack to wait */
    }

    /**
    * Low level native method to open the serial port
    */
    private native int init();

    /**
    * Low level native method to close the serial port
    */
    private native int quit(int fd);

    /**
    * Low level native method to send the command across the serial port
    */
    private native int send(int fd, String message);

    static
    {
        System.loadLibrary("serialport");
    }
};
